#ifndef _ECTOR_GL_BUFFER_BASE_EO_H_
#define _ECTOR_GL_BUFFER_BASE_EO_H_

#ifndef _ECTOR_GL_BUFFER_BASE_EO_CLASS_TYPE
#define _ECTOR_GL_BUFFER_BASE_EO_CLASS_TYPE

typedef Eo Ector_GL_Buffer_Base;

#endif

#ifndef _ECTOR_GL_BUFFER_BASE_EO_TYPES
#define _ECTOR_GL_BUFFER_BASE_EO_TYPES


#endif
/** Ector GL buffer base class
 *
 * @ingroup Ector_GL_Buffer_Base
 */
#define ECTOR_GL_BUFFER_BASE_MIXIN ector_gl_buffer_base_mixin_get()

EWAPI const Efl_Class *ector_gl_buffer_base_mixin_get(void);

/**
 * @brief GL texture ID
 *
 * @return GL texture ID
 *
 * @ingroup Ector_GL_Buffer_Base
 */
EOAPI int ector_gl_buffer_base_texture_get(const Eo *obj);

/**
 * @brief Framebuffer object ID
 *
 * @return GL framebuffer ID, 0 if there is no FBO
 *
 * @ingroup Ector_GL_Buffer_Base
 */
EOAPI int ector_gl_buffer_base_fbo_get(const Eo *obj);

/**
 * @brief If @c true, the image is covering the entire GL texture, ie. it's not
 * part of an atlas.
 *
 * @return @c true if the image is covering the whole GL texture, @c false
 * otherwise
 *
 * @ingroup Ector_GL_Buffer_Base
 */
EOAPI Eina_Bool ector_gl_buffer_base_whole_get(const Eo *obj);

/**
 * @brief Returns the texture vertices to draw this image with no rotation
 *
 * The 4 points are then defined as (x,y), (x+w,y), (x,y+h), (x+w,y+h).
 *
 * @param[out] x X position of this image inside the texture atlas, from 0 to 1
 * @param[out] y Y position of this image inside the texture atlas, from 0 to 1
 * @param[out] w Width of this image inside the texture atlas, from 0 to 1
 * @param[out] h Height of this image inside the texture atlas, from 0 to 1
 *
 * @ingroup Ector_GL_Buffer_Base
 */
EOAPI void ector_gl_buffer_base_vertices_get(const Eo *obj, double *x, double *y, double *w, double *h);

#ifdef ECTOR_GL_BUFFER_BASE_PROTECTED
/**
 * @brief Attach to an existing texture (or FBO). Used from child classes.
 *
 * @param[in] texid GL texture ID
 * @param[in] fboid Framebuffer object ID
 * @param[in] cspace Colorspace
 * @param[in] imw Image width
 * @param[in] imh Image height
 * @param[in] tx Texture X coordinate
 * @param[in] ty Texture Y coordinate
 * @param[in] tw Texture width
 * @param[in] th Texture height
 * @param[in] l Left padding
 * @param[in] r Right padding
 * @param[in] t Top padding
 * @param[in] b Bottom padding
 *
 * @ingroup Ector_GL_Buffer_Base
 */
EOAPI void ector_gl_buffer_base_attach(Eo *obj, int texid, int fboid, Efl_Gfx_Colorspace cspace, int imw, int imh, int tx, int ty, int tw, int th, int l, int r, int t, int b);
#endif

#endif
